﻿using Microsoft.Extensions.Logging;
using ProjectSellPC.Repos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectSellPC.DocBuilder
{
    internal class TableReport
    {
        private readonly IChequeRepository _chequeRepository;
        private readonly IProductRepository _productRepository;
        private readonly ILogger<TableReport> _logger;

        internal static readonly string[] Headers = { "Клиент", "Дата", "Количество" };

        public TableReport(IChequeRepository chequeRepository, IProductRepository productRepository, ILoggerFactory loggerFactory)
        {
            _chequeRepository = chequeRepository ?? throw new ArgumentNullException(nameof(chequeRepository));
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
            _logger = loggerFactory.CreateLogger<TableReport>();
        }

        public bool CreateTable(string filePath, int productId, DateTime startDate, DateTime endDate)
        {
            try
            {
                new ExcelBuilder(filePath)
                    .AddHeader("Отчет по продажам товара", 0, 3)
                    .AddParagraph($"за период с {startDate.ToShortDateString()} по {endDate.ToShortDateString()}", 0)
                    .AddTable(new[] { 20, 20, 15 }, GetData(productId, startDate, endDate))
                    .Build();

                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании отчета");
                return false;
            }
        }

        private List<string[]> GetData(int productId, DateTime startDate, DateTime endDate)
        {
            // Получение данных о чеках, содержащих указанный товар
            var checksWithProduct = _chequeRepository
                .ReadAll()
                .Where(c => c.PurchaseDate >= startDate && c.PurchaseDate <= endDate
                            && c.Products.Any(p => p.ProductID == productId))
                .Select(c => new
                {
                    ClientName = c.Client?.ToString() ?? "Неизвестно",
                    Date = c.PurchaseDate,
                    Quantity = c.Products
                        .Where(p => p.ProductID == productId)
                        .Sum(p => p.Count)
                })
                .OrderBy(x => x.Date);

            // Формирование итоговой таблицы
            return new List<string[]> { Headers }
                .Union(
                    checksWithProduct.Select(x => new string[]
                    {
                    x.ClientName,
                    x.Date.ToShortDateString(),
                    x.Quantity.ToString()
                    })
                )
                .Union(new[]
                {
                new string[]
                {
                    "Всего",
                    "",
                    checksWithProduct.Sum(x => x.Quantity).ToString()
                }
                })
                .ToList();
        }
    }
}
