﻿using Dapper;
using Microsoft.Extensions.Logging;
using Npgsql;
using ProjectSellPC.Entites;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectSellPC.Repos.Impements
{
    public class ProductInChequeRepo : IProductInChequeRepository
    {

        private readonly IProductRepository _productRepository;
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ProductInChequeRepo> _logger;

        public ProductInChequeRepo(IConnectionString connectionString, ILoggerFactory loggerFactory, IProductRepository productRepository)
        {
            _connectionString = connectionString;
            _logger = loggerFactory.CreateLogger<ProductInChequeRepo>();
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
        }

        private IDbConnection CreateConnection() => new NpgsqlConnection(_connectionString.ConnectionString);

        public void Create(ProductInCheque productInCheque)
        {
            _logger.LogInformation("Добавление товара в чек");
            _logger.LogDebug("Товар в чеке: {json}", Newtonsoft.Json.JsonConvert.SerializeObject(productInCheque));

            using (var connection = CreateConnection())
            {
                connection.Open();
                using (var transaction = connection.BeginTransaction())
                {
                    try
                    {
                        var sql = "INSERT INTO \"productincheque\" (\"productid\", \"count\") " +
                                  "VALUES (@ProductID, @Count) RETURNING \"ID\"";

                        productInCheque.ID = connection.ExecuteScalar<int>(sql, new
                        {
                            ProductID = productInCheque.ProductID,
                            Count = productInCheque.Count
                        }, transaction);

                        transaction.Commit();
                    }
                    catch (Exception ex)
                    {
                        transaction.Rollback();
                        _logger.LogError(ex, "Ошибка при добавлении товара в чек");
                        throw;
                    }
                }
            }
        }

        public ProductInCheque Read(int id)
        {
            _logger.LogInformation("Получение товара в чеке по ID");
            _logger.LogDebug("ID товара в чеке: {id}", id);

            using (var connection = CreateConnection())
            {
                try
                {
                    var sql = "SELECT * FROM \"productincheque\" WHERE \"id\" = @ID";
                    var productInCheque = connection.QuerySingleOrDefault<ProductInCheque>(sql, new { ID = id });
                    if (productInCheque != null)
                    {
                        // Загрузка дополнительной информации о продукте при необходимости
                        var product = _productRepository.Read(productInCheque.ProductID);
                        productInCheque.ProductID = product?.ID ?? 0;
                    }
                    return productInCheque;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при получении товара в чеке");
                    throw;
                }
            }
        }

        public IEnumerable<ProductInCheque> ReadAll()
        {
            _logger.LogInformation("Получение всех товаров в чеках");

            using (var connection = CreateConnection())
            {
                try
                {
                    var sql = "SELECT * FROM \"productincheque\"";
                    var productsInCheque = connection.Query<ProductInCheque>(sql).ToList();

                    foreach (var productInCheque in productsInCheque)
                    {
                        var product = _productRepository.Read(productInCheque.ProductID);
                        productInCheque.ProductID = product?.ID ?? 0;
                    }
                    return productsInCheque;
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка при получении списка товаров в чеках");
                    throw;
                }
            }
        }

        public void Delete(int id)
        {
            _logger.LogInformation("Удаление товара из чека");
            _logger.LogDebug("ID товара в чеке: {id}", id);

            try
            {
                using (var connection = CreateConnection())
                {
                    var sql = "DELETE FROM \"productincheque\" WHERE \"id\" = @ID";
                    connection.Execute(sql, new { ID = id });
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении товара из чека");
                throw;
            }
        }
    }
}
